#! /usr/bin/env lua

--[[
	TreeSortable example. Another nice (and resource hungry) feature.
	Sorting a model requires ~= N * log(N) calls to the comparison function,
	where N is the number of rows of the model.
	
	This is only an example, the custom sorting function doesn't have more
	precision than setting the column type as gdouble and using the default
	sorting function.
--]]

require("lgui")

local window = lgui.Window.new()
local view	 = lgui.TreeView.new()
local model	 = lgui.ListStore.new("gchararray", "gboolean")
local col1	 = lgui.TreeViewColumn.new("Precision numbers", lgui.CellRendererText.new(), "text", 0)
local col2	 = lgui.TreeViewColumn.new("Flag", lgui.CellRendererToggle.new(), "active", 1)
local iter	 = lgui.TreeIter.new()

col1:setSortColumnId(0)
col2:setSortColumnId(1)

for i = 1, 2000 do
	local j = math.random() * 50
	model:append(iter)
	model:seto(iter, j, j > 25)
end

view:appendColumn(col1, col2)
view:set("model", model)

local getm = lgui.TreeModel.get

-- Configure the sorting function for column 0
local function sort(data, iterA, iterB)
	local a, b = getm(model, iterA, 0), getm(model, iterB, 0)
	return tonumber(a) > tonumber(b) and 1 or - 1
end

lgui.TreeSortable.setSortFunc(model, 0, sort)
local scroll = lgui.ScrolledWindow.new()
scroll:add(view)
window:add(scroll)
window:set("window-position", lgui.WIN_POS_CENTER, "title", "TreeSortable demo",
	"width-request", 300, "height-request", 400)
window:connect("delete-event", lgui.quit)
window:showAll()

lgui.main()
