/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "container.h"

/* GtkContainer */

int lgui_container_child_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int i, size = lua_gettop(L);
	Object* con = lua_touserdata(L, 1);
	Object* obj = lua_touserdata(L, 2);
	const char* property = NULL;

	/* Set the properties, pair by pair */
	for(i = 3; i <= size; i += 2)
	{
		/* Get the property */
		luaL_checktype(L, i, LUA_TSTRING);
		property = lua_tostring(L, i);
		
		/* Set the value */
		GParamSpec* spec = gtk_container_class_find_child_property(G_OBJECT_GET_CLASS(con->pointer), property);
	
		if(!spec)
			luaL_error(L, "The GObject doesn't have the property %s!", property);
		
		GValue val = {0};
		g_value_init(&val, spec->value_type);
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValueSet");
		lua_pushlightuserdata(L, &val);
		lua_pushvalue(L, i + 1);
		lua_call(L, 2, 0);
		gtk_container_child_set_property(GTK_CONTAINER(con->pointer), GTK_WIDGET(obj->pointer), property, &val);
		g_value_unset(&val);
	}

	return 0;
}

int lgui_container_child_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int i, size = lua_gettop(L);
	Object* con = lua_touserdata(L, 1);
	Object* obj = lua_touserdata(L, 2);
	const char* property = NULL;

	for(i = 3; i <= size; ++i)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		property = lua_tostring(L, i);
		
		/* Get the value */
		GParamSpec* spec = gtk_container_class_find_child_property(G_OBJECT_GET_CLASS(con->pointer), property);
	
		if(!spec)
			luaL_error(L, "The GObject doesn't have the property %s!", property);
		
		GValue val = {0};
		g_value_init(&val, spec->value_type);
		gtk_container_child_get_property(GTK_CONTAINER(con->pointer), GTK_WIDGET(obj->pointer), property, &val);
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValuePush");
		lua_pushlightuserdata(L, &val);
		lua_call(L, 1, 1);
		g_value_unset(&val);
	}

	return i - 3;
}

int lgui_container_add(lua_State* L)
{
	/* Get the container */
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* container = lua_touserdata(L, 1);

	int top = lua_gettop(L), i;
	Object* children = NULL;

	/* Add widget by widget */
	for(i = 2; i <= top; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		children = lua_touserdata(L, i);;
		gtk_container_add(GTK_CONTAINER(container->pointer), GTK_WIDGET(children->pointer));
	}

	return 0;
}

int lgui_container_remove(lua_State* L)
{
	/* Get the container */
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* container = lua_touserdata(L, 1);

	int top = lua_gettop(L), i;
	Object* children = NULL;

	/* Remove widget by widget */
	for(i = 2; i <= top; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		children = lua_touserdata(L, i);;
		gtk_container_remove(GTK_CONTAINER(container->pointer), GTK_WIDGET(children->pointer));
	}

	return 0;
}

int lgui_container_get_children(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GList* children = gtk_container_get_children(GTK_CONTAINER(obj->pointer));
	lua_pushliteral(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, children);
	lua_pushinteger(L, TYPE_OBJECT);
	lua_call(L, 2, 1);
	
	g_list_free(children);
	
	return 1;
}

int lgui_container_set_reallocate_redraws(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_container_set_reallocate_redraws(GTK_CONTAINER(obj->pointer), 
		lua_toboolean(L, 2));

	return 0;
}
