/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.

    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "gdk.h"

/* Events */

int lgui_event_button_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	GdkEventButton* eb = (GdkEventButton*)obj->pointer;

	lua_pushnumber(L, eb->type);
	lua_pushnumber(L, eb->button);
	lua_pushnumber(L, eb->state);
	lua_pushnumber(L, eb->x);
	lua_pushnumber(L, eb->y);

	return 5;
}

int lgui_event_key_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	GdkEventKey* eb = (GdkEventKey*)obj->pointer;

	lua_pushnumber(L, eb->type);
	lua_pushnumber(L, eb->state);
	lua_pushnumber(L, eb->keyval);

	return 3;
}

/* Atom. Converts a pointer to lua_Integer. Evil, but is almost impossible
 * to a GdkAtom overflow this... */

int lgui_atom_intern(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GdkAtom atom = gdk_atom_intern(lua_tostring(L, 1), FALSE);
	lua_pushinteger(L, (lua_Integer)atom);

	return 1;
}

int lgui_atom_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	GdkAtom atom = (GdkAtom)lua_tointeger(L, 1);

	gchar* str = gdk_atom_name(atom);
	lua_pushstring(L, str);
	g_free(str);

	return 1;
}

/* GdkCursor */

static int priv_gdk_cursor_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting GdkCursor!");
	#endif

	Object* obj = lua_touserdata(L, 1);
	gdk_cursor_unref((GdkCursor*)obj->pointer);

	return 0;
}

int lgui_gdk_cursor_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);

	#ifdef IDEBUG
	g_debug("Creating a new GdkCursor!");
	#endif

	Object* obj = lua_newuserdata(L, sizeof(Object));
	obj->pointer = gdk_cursor_new((GdkCursorType)lua_tointeger(L, 1));
	obj->need_unref = TRUE; /* Not used, just for consistency */
	int top = lua_gettop(L);

	/* Shared metatable */
	if(luaL_newmetatable(L, "__lguiCuRsOrMt"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "lgui");
		lua_pushliteral(L, "GdkCursor");
		lua_pushcfunction(L, priv_gdk_cursor_gc);
		lua_call(L, 4, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);

	return 1;
}

/* GdkWindow */

int lgui_gdk_window_set_cursor(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);

	if(lua_isnil(L, 2))
	{
		gdk_window_set_cursor(GDK_WINDOW(obj1->pointer), NULL);
	}
	else
	{
		luaL_checktype(L, 2, LUA_TUSERDATA);
		Object* obj2 = lua_touserdata(L, 2);
		gdk_window_set_cursor(GDK_WINDOW(obj1->pointer), (GdkCursor*)obj2->pointer);
	}

	return 0;
}

int lgui_gdk_window_get_pointer(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gint posX, posY;
	GdkModifierType mod;

	GdkWindow* res = gdk_window_get_pointer(GDK_WINDOW(obj->pointer),
		&posX, &posY, &mod);

	object_new(L, res, FALSE);
	lua_pushinteger(L, posX);
	lua_pushinteger(L, posY);
	lua_pushinteger(L, (gint)mod);

	return 4;
}

int lgui_gdk_window_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gdk_window_clear(GDK_WINDOW(obj->pointer));

	return 0;
}

int lgui_gdk_window_clear_area(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	luaL_checktype(L, 5, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);

	gdk_window_clear_area(GDK_WINDOW(obj->pointer), lua_tointeger(L, 2),
		lua_tointeger(L, 3), lua_tointeger(L, 4), lua_tointeger(L, 5));

	return 0;
}

/* General */

int lgui_gdk_beep(lua_State* L)
{
	gdk_beep();
	return 0;
}
