/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "menu.h"

/* MenuShell */

int lgui_menu_shell_append(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* menu = lua_touserdata(L, 1);
	int i, size = lua_gettop(L);

	for(i = 2; i <= size; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* item = lua_touserdata(L, i);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu->pointer), GTK_WIDGET(item->pointer));
	}

	return 0;
}

int lgui_menu_shell_insert(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_menu_shell_insert(GTK_MENU_SHELL(obj1->pointer), GTK_WIDGET(obj2->pointer),
		lua_tonumber(L, 3));

	return 0;
}

/* GtkMenuBar */

int lgui_menu_bar_new(lua_State* L)
{
	GtkWidget* ptr = gtk_menu_bar_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

/* GtkMenu */

int lgui_menu_new(lua_State* L)
{
	GtkWidget* ptr = gtk_menu_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_menu_popup(lua_State* L)
{
	/* Simplify the task! Very incomplete. */
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_menu_popup(GTK_MENU(obj->pointer), NULL, NULL, NULL, NULL,
	0, gtk_get_current_event_time());

	return 0;
}

int lgui_menu_popup_from_status_icon(lua_State* L)
{
	/* Lazy */
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_menu_popup(GTK_MENU(obj1->pointer), NULL, NULL, gtk_status_icon_position_menu,
		obj2->pointer, 0, gtk_get_current_event_time());
		
	return 0;
}

/* GtkMenuItem */

int lgui_menu_item_new(lua_State* L)
{
	GtkWidget* ptr = gtk_menu_item_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}


int lgui_menu_item_new_with_mnemonic(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_menu_item_new_with_mnemonic(lua_tostring(L, 1));
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_menu_item_set_submenu(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* menu = lua_touserdata(L, 1);
	Object* submenu = lua_touserdata(L, 2);

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu->pointer), GTK_WIDGET(submenu->pointer));

	return 0;
}

/* GtkImageMenuItem */

int lgui_image_menu_item_new(lua_State* L)
{
	GtkWidget* ptr = gtk_image_menu_item_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_image_menu_item_new_from_stock(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_image_menu_item_new_from_stock(lua_tostring(L, 1), NULL);
	object_new(L, ptr, FALSE);
	
	return 1;
}

/* GtkCheckMenuItem */

int lgui_check_menu_item_new_with_mnemonic(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_check_menu_item_new_with_mnemonic(lua_tostring(L, 1));
	object_new(L, ptr, FALSE);
	
	return 1;
}

/* GtkTearoffMenuItem */

int lgui_tearoff_menu_item_new(lua_State* L)
{
	GtkWidget* ptr = gtk_tearoff_menu_item_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

/* GtkSeparatorMenuItem */

int lgui_separator_menu_item_new(lua_State* L)
{
	GtkWidget* ptr = gtk_separator_menu_item_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}
