/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "pango.h"

static int priv_pango_font_description_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a PangoFontDescription!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);
	pango_font_description_free(obj->pointer);

	return 0;
}

static int priv_pango_font_description_to_string(lua_State* L)
{
	Object* obj = lua_touserdata(L, 1);

	gchar* str = pango_font_description_to_string(obj->pointer);
	
	if(str)
	{
		lua_pushstring(L, str);
		g_free(str);
	}
	else
		lua_pushstring(L, "");
		
	return 1;
}

int lgui_pango_font_description_from_string(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	const gchar* str = lua_tostring(L, 1);

	#ifdef IDEBUG
	g_debug("Creating a new PangoFontDescription!");
	#endif

	Object* obj = lua_newuserdata(L, sizeof(Object));
	obj->pointer = pango_font_description_from_string(str);
	obj->need_unref = TRUE; /* Not used, just for consistency */
	int top = lua_gettop(L);
	
	/* Shared metatable */
	if(luaL_newmetatable(L, "__lguiPaNgOFdMt"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "lgui");
		lua_pushliteral(L, "PangoFontDescription");
		lua_pushcfunction(L, priv_pango_font_description_gc);
		lua_pushcfunction(L, priv_pango_font_description_to_string);
		lua_call(L, 5, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);

	return 1;
}

int lgui_pango_font_description_set_size(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_font_description_set_size(obj->pointer, lua_tointeger(L, 2));

	return 0;
}

int lgui_pango_layout_set_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	int top = lua_gettop(L);
	int length = -1;

	if(top >= 3)
	{
		luaL_checktype(L, 3, LUA_TNUMBER);
		length = lua_tointeger(L, 3);
	}

	Object* obj = lua_touserdata(L, 1);
	const gchar* str = lua_tostring(L, 2);

	pango_layout_set_text(PANGO_LAYOUT(obj->pointer), str, length);

	return 0;
}

int lgui_pango_layout_set_markup(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	int top = lua_gettop(L);
	int length = -1;

	if(top >= 3)
	{
		luaL_checktype(L, 3, LUA_TNUMBER);
		length = lua_tointeger(L, 3);
	}

	Object* obj = lua_touserdata(L, 1);
	const gchar* str = lua_tostring(L, 2);

	pango_layout_set_markup(PANGO_LAYOUT(obj->pointer), str, length);

	return 0;
}

int lgui_pango_layout_set_font_description(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	pango_layout_set_font_description(PANGO_LAYOUT(obj1->pointer), obj2->pointer);
	return 0;
}

int lgui_pango_layout_set_width(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_width(PANGO_LAYOUT(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_pango_layout_set_height(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_height(PANGO_LAYOUT(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_pango_layout_set_wrap(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_wrap(PANGO_LAYOUT(obj->pointer), lua_toboolean(L, 2));

	return 0;
}

int lgui_pango_layout_set_spacing(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_spacing(PANGO_LAYOUT(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_pango_layout_set_indent(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_indent(PANGO_LAYOUT(obj->pointer), lua_tointeger(L, 2));

	return 0;
}
int lgui_pango_layout_set_justify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_justify(PANGO_LAYOUT(obj->pointer), lua_toboolean(L, 2));

	return 0;
}

int lgui_pango_layout_set_aligment(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_alignment(PANGO_LAYOUT(obj->pointer), (PangoAlignment)lua_tointeger(L, 2));

	return 0;
}

int lgui_pango_layout_set_ellipsize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	pango_layout_set_ellipsize(PANGO_LAYOUT(obj->pointer), (PangoEllipsizeMode)lua_tointeger(L, 2));

	return 0;
}
