/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "treesortable.h"

static gint priv_sort_callback(GtkTreeModel* model, GtkTreeIter* a, 
	GtkTreeIter* b, Data* data)
{
	lua_State* L = data->L;
	int size = lua_gettop(L);

	/* Push the function and user data */
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->function_ref);
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->ud_ref);

	/* Push the iters */
	struct_new(L, a, FALSE);
	struct_new(L, b, FALSE);

	/* Call the function */
	lua_call(L, 3, 1);

	/* Get the return */
	gint res = lua_tointeger(L, -1);
	lua_settop(L, size);

	#ifdef IDEBUG
	g_debug("SortCallback returning!");
	#endif

	return res;
}

int lgui_tree_sortable_set_sort_column_id(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(obj->pointer), 
		lua_tointeger(L, 2), lua_tointeger(L, 3));
		
	return 0;
}

int lgui_tree_sortable_get_sort_column_id(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	
	gint id;
	GtkSortType order;
	
	Object* obj = lua_touserdata(L, 1);
	gboolean res = gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(obj->pointer), 
		&id, &order);
		
	lua_pushboolean(L, res);
	lua_pushinteger(L, id);	
	lua_pushinteger(L, order);	
		
	return 3;
}

int lgui_tree_sortable_set_sort_func(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TFUNCTION);

	int size = lua_gettop(L);
	int ud_ref = -1;
	Object* obj = lua_touserdata(L, 1);

	/* User data? */
	if(size > 3)
	{
		/* Create a reference to the user data */
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}

	lua_pushvalue(L, 3);
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);

	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;

	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(obj->pointer), lua_tointeger(L, 2),
	(GtkTreeIterCompareFunc)priv_sort_callback, data, callback_free);

	#ifdef IDEBUG
	g_debug("SortFunc setted!");
	#endif

	return 0;
}
