/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/


#include "treestore.h"

int lgui_tree_store_new(lua_State* L)
{
	int size = lua_gettop(L), i;
	GType* types = g_malloc(sizeof(GType) * size);

	/* Fill the types array */
	for(i = 1; i <= size; i++)
		types[i - 1] = g_type_from_name(luaL_checkstring(L, i));

	GtkTreeStore* ptr = gtk_tree_store_newv(size, types);
	object_new(L, ptr, TRUE);
	g_free(types);
	
	return 1;
}

int lgui_tree_store_insert(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 4, LUA_TNUMBER);
	
	Object* tree = lua_touserdata(L, 1);
	Object* iter = lua_touserdata(L, 2);
	int type = lua_type(L, 3);
	GtkTreeIter* parent = NULL;	
	
	/* If it's a iterator, else let it be NULL */
	if(type == LUA_TUSERDATA)
	{
		Object* pud = lua_touserdata(L, 3);
		parent = (GtkTreeIter*)pud->pointer;
	}
	
	gtk_tree_store_insert(GTK_TREE_STORE(tree->pointer), (GtkTreeIter*)iter->pointer, parent, lua_tointeger(L, 4));
	
	return 0;	
}

int lgui_tree_store_append(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* tree = lua_touserdata(L, 1);
	Object* iter = lua_touserdata(L, 2);
	int type = lua_type(L, 3);
	GtkTreeIter* parent = NULL;	
	
	/* If it's a iterator, else let it be NULL */
	if(type == LUA_TUSERDATA)
	{
		Object* pud = lua_touserdata(L, 3);
		parent = (GtkTreeIter*)pud->pointer;
	}
	
	gtk_tree_store_append(GTK_TREE_STORE(tree->pointer), (GtkTreeIter*)iter->pointer, parent);
	
	return 0;	
}

int lgui_tree_store_prepend(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* tree = lua_touserdata(L, 1);
	Object* iter = lua_touserdata(L, 2);
	int type = lua_type(L, 3);
	GtkTreeIter* parent = NULL;	
	
	/* If it's a iterator, else let it be NULL */
	if(type == LUA_TUSERDATA)
	{
		Object* pud = lua_touserdata(L, 3);
		parent = (GtkTreeIter*)pud->pointer;
	}
	
	gtk_tree_store_prepend(GTK_TREE_STORE(tree->pointer), (GtkTreeIter*)iter->pointer, parent);
	
	return 0;	
}

int lgui_tree_store_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_tree_store_clear(GTK_TREE_STORE(obj->pointer));
	
	return 0;
}

int lgui_tree_store_remove(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* tree = lua_touserdata(L, 1);
	Object* iter = lua_touserdata(L, 2);
	
	gtk_tree_store_remove(GTK_TREE_STORE(tree->pointer), (GtkTreeIter*)iter->pointer);
	
	return 0;	
}

int lgui_tree_store_swap(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* tree = lua_touserdata(L, 1);
	Object* iter1 = lua_touserdata(L, 2);
	Object* iter2 = lua_touserdata(L, 3);
	
	gtk_tree_store_swap(GTK_TREE_STORE(tree->pointer), (GtkTreeIter*)iter1->pointer, (GtkTreeIter*)iter2->pointer);
	
	return 0;	
}

int lgui_tree_store_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i, curr;
	GtkTreeStore* tree = GTK_TREE_STORE(((Object*)lua_touserdata(L, 1))->pointer);
	GtkTreeIter*  iter = (GtkTreeIter*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValueSet");

	/* {model, iter, key, val, ...} */
	int nvals = (size - 2) / 2;
	 
	/* Allocate the temp GValue array  */
	int vsize = sizeof(GValue) * nvals;
	GValue* vals = g_malloc(vsize);
	GValue* val;
	memset(vals, 0, vsize);
	
	/* Alocate the columns array */
	int* cols = g_malloc(sizeof(int) * nvals);
 	
	/* Get the fields by field */
	for(i = 3; i <= size; i += 2)
	{
		luaL_checknumber(L, i);
		luaL_checkany(L, i + 1);

		int column = lua_tointeger(L, i);
		GType type = gtk_tree_model_get_column_type(GTK_TREE_MODEL(tree), column);

		#ifdef IDEBUG
		g_debug("Setting TreeStore column %i, of type %s!", column, g_type_name(type));
		#endif
		
		/* Current pair */
		curr = (i - 3) / 2;
		cols[curr] = column;
		val = &vals[curr];
		
		g_value_init(val, type);
		lua_pushvalue(L, size + 1); /* "__lobjValueSet" */
		lua_pushlightuserdata(L, val);
		lua_pushvalue(L, i + 1);
		lua_call(L, 2, 0);
	}
	
	/* Ok, now we have the array with the values. Set them with one hit! */
	gtk_tree_store_set_valuesv(tree, iter, cols, vals, nvals);
	
	/* Now unset them all */
	for(i = 0; i < nvals; ++i)
		g_value_unset(&vals[i]);
		
	/* Free the dinamic alocated arrays */
	g_free(vals);
	g_free(cols);

	return 0;
}

int lgui_tree_store_seto(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i;
	GtkTreeStore* tree = GTK_TREE_STORE(((Object*)lua_touserdata(L, 1))->pointer);
	GtkTreeIter*  iter = (GtkTreeIter*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValueSet");

	/* {model, iter, val, val, ...} */
	int nvals = (size - 2);
	 
	/* Alocate the temp GValue array  */
	int vsize = sizeof(GValue) * nvals;
	GValue* vals = g_malloc(vsize);
	GValue* val;
	memset(vals, 0, vsize);
	
	/* Allocate the columns array */
	int* cols = g_malloc(sizeof(int) * nvals);
 	
	/* Get the fields by field */
	for(i = 3; i <= size; ++i)
	{
		int column = i - 3;
		GType type = gtk_tree_model_get_column_type(GTK_TREE_MODEL(tree), column);

		#ifdef IDEBUG
		g_debug("Setting listStore column %i, of type %s!", column, g_type_name(type));
		#endif
		
		/* Current pair */
		cols[column] = column;
		val = &vals[column];
		
		g_value_init(val, type);
		lua_pushvalue(L, size + 1); /* "__lobjValueSet" */
		lua_pushlightuserdata(L, val);
		lua_pushvalue(L, i);
		lua_call(L, 2, 0);
	}
	
	/* Ok, now we have the array with the values. Set them with one hit! */
	gtk_tree_store_set_valuesv(tree, iter, cols, vals, nvals);
	
	/* Now unset them all */
	for(i = 0; i < nvals; ++i)
		g_value_unset(&vals[i]);
		
	/* Free the dinamic alocated arrays */
	g_free(vals);
	g_free(cols);

	return 0;
}

int lgui_tree_store_move_before(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_tree_store_move_before(GTK_TREE_STORE(obj1->pointer),
		(GtkTreeIter*)obj2->pointer, (GtkTreeIter*)obj3->pointer);
	
	return 0;
}

int lgui_tree_store_move_after(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_tree_store_move_after(GTK_TREE_STORE(obj1->pointer),
		(GtkTreeIter*)obj2->pointer, (GtkTreeIter*)obj3->pointer);
	
	return 0;
}
