/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.

    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "widget.h"

int lgui_widget_realize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_realize(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_activate(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_activate(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_show(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_show(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_show_now(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_show_now(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_show_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_show_all(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_hide(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_hide(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_hide_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_hide_all(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_focus(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_grab_focus(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_get_window(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	object_new(L, GTK_WIDGET(obj->pointer)->window, FALSE);
	return 1;
}

int lgui_widget_get_size(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	lua_pushnumber(L, GTK_WIDGET(obj->pointer)->allocation.width);
	lua_pushnumber(L, GTK_WIDGET(obj->pointer)->allocation.height);

	return 2;
}

int lgui_widget_get_pos(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	lua_pushnumber(L, GTK_WIDGET(obj->pointer)->allocation.x);
	lua_pushnumber(L, GTK_WIDGET(obj->pointer)->allocation.y);

	return 2;
}

int lgui_widget_queue_draw(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_widget_queue_draw(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_queue_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_widget_queue_clear(GTK_WIDGET(obj->pointer));

	return 0;
}



int lgui_widget_add_events(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_widget_add_events(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_widget_set_events(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_widget_set_events(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_widget_get_pointer(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gint p1, p2;

	gtk_widget_get_pointer(GTK_WIDGET(obj->pointer), &p1, &p2);
	lua_pushnumber(L, p1);
	lua_pushnumber(L, p2);

	return 2;
}

int lgui_widget_render_icon(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);

	int top = lua_gettop(L);
	char* func = NULL;
	Object* obj = lua_touserdata(L, 1);

	if(top > 3)
	{
		luaL_checktype(L, 4, LUA_TSTRING);
		func = (char*)lua_tostring(L, 4);
	}

	GdkPixbuf* buf = gtk_widget_render_icon(GTK_WIDGET(obj->pointer), lua_tostring(L, 2),
		(GtkIconSize)lua_tointeger(L, 3), func);

	object_new(L, buf, TRUE);
	return 1;
}

int lgui_widget_modify_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	int top = lua_gettop(L);
	GdkColor* color = NULL;

	Object* obj = lua_touserdata(L, 1);

	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		Object* col = lua_touserdata(L, 3);
		color = (GdkColor*)col->pointer;
	}

	gtk_widget_modify_text(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2), color);

	return 0;
}

int lgui_widget_modify_fg(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	int top = lua_gettop(L);
	GdkColor* color = NULL;

	Object* obj = lua_touserdata(L, 1);

	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		Object* col = lua_touserdata(L, 3);
		color = (GdkColor*)col->pointer;
	}

	gtk_widget_modify_fg(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2), color);

	return 0;
}

int lgui_widget_modify_bg(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	int top = lua_gettop(L);
	GdkColor* color = NULL;

	Object* obj = lua_touserdata(L, 1);

	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		Object* col = lua_touserdata(L, 3);
		color = (GdkColor*)col->pointer;
	}

	gtk_widget_modify_bg(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2), color);

	return 0;
}

int lgui_widget_modify_base(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	int top = lua_gettop(L);
	GdkColor* color = NULL;

	Object* obj = lua_touserdata(L, 1);

	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		Object* col = lua_touserdata(L, 3);
		color = (GdkColor*)col->pointer;
	}

	gtk_widget_modify_base(GTK_WIDGET(obj->pointer), lua_tointeger(L, 2), color);

	return 0;
}

int lgui_widget_modify_font(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	int top = lua_gettop(L);
	PangoFontDescription* desc = NULL;

	Object* obj = lua_touserdata(L, 1);

	if(top > 1)
	{
		luaL_checktype(L, 2, LUA_TUSERDATA);
		Object* pbj = lua_touserdata(L, 2);
		desc = (PangoFontDescription*)pbj->pointer;
	}

	gtk_widget_modify_font(GTK_WIDGET(obj->pointer), desc);

	return 0;
}

/* Evil */
int lgui_widget_get_handle(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	lua_pushinteger(L, GET_HANDLER(GTK_WIDGET(obj->pointer)->window));

	return 1;
}

int lgui_widget_thaw_child_notify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_thaw_child_notify(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_freeze_child_notify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_widget_freeze_child_notify(GTK_WIDGET(obj->pointer));

	return 0;
}

int lgui_widget_child_notify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_widget_child_notify(GTK_WIDGET(obj->pointer), lua_tostring(L, 2));

	return 0;
}


int lgui_widget_get_snapshot(lua_State* L)
{
#if GTK_MINOR_VERSION >= 13
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	int top = lua_gettop(L);
	GdkRectangle* rect;
		
	if(top >= 5)
	{
		/* Get the rectangle */
		luaL_checktype(L, 2, LUA_TNUMBER);
		luaL_checktype(L, 3, LUA_TNUMBER);
		luaL_checktype(L, 4, LUA_TNUMBER);
		luaL_checktype(L, 5, LUA_TNUMBER);
		rect = g_malloc(sizeof(GdkRectangle));
		rect->x = lua_tointeger(L, 2);
		rect->y = lua_tointeger(L, 3);
		rect->width = lua_tointeger(L, 4);
		rect->height = lua_tointeger(L, 5);
	}
	else
		rect = NULL;
		
	GdkPixmap* pix = gtk_widget_get_snapshot(GTK_WIDGET(obj->pointer), rect);
	
	if(pix)
	{
		GdkPixbuf* buf = gdk_pixbuf_get_from_drawable(NULL, GDK_DRAWABLE(pix), 
		gdk_colormap_get_system(), 0, 0, 0, 0, -1, -1);
		g_object_unref(pix);
		object_new(L, buf, TRUE);
	}
	else
		lua_pushnil(L);
		
	/* Release the resources */
	if(rect) g_free(rect);
#else
	lua_pushnil(L);
#endif
	
	return 1;
}

int lgui_widget_get_toplevel(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_widget_get_toplevel(GTK_WIDGET(obj->pointer));
	object_new(L, ptr, FALSE);
	
	return 1;
}
